/****h* ROBODoc/Utilities
 * FUNCTION
 *   Set of general purpose utility functions.
 *****
 * $Id: util.c,v 1.23 2003/11/02 19:04:14 gumpu Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>             /* for RB_Say() */
#include <time.h>
#include <assert.h>

#include "robodoc.h"
#include "globals.h"
#include "links.h"
#include "headers.h"
#include "folds.h"
#include "items.h"
#include "util.h"


#ifdef DMALLOC
#include <dmalloc.h>
#endif

/****f* Utilities/RB_FilePart [2.0x]
 * NAME
 *   RB_FilePart
 * SYNOPSIS
 *   char *RB_FilePart(char *file_name)
 * FUNCTION
 *   return the basename (like Amiga/Dos/FilePart())
 * NOTES
 *   koessi
 * SEE ALSO
 * SOURCE
 */

char               *
RB_FilePart( char *file_name )
{
    char               *cur_char;
    char                c;

    if ( ( cur_char = file_name ) != NULL )
    {
        for ( ; ( c = *cur_char ) != '\0'; ++cur_char )
        {
            if ( ( c == '/' ) || ( c == ':' ) )
            {
                ++cur_char;
                while ( '/' == *cur_char )
                {
                    ++cur_char;
                }

                if ( *cur_char )
                {
                    file_name = cur_char;
                }
            }
        }
    }
    return ( file_name );
}

/*** RB_File_Part ***/

#if 0
/****f* Utilities/RB_Has_Remark_Marker
 * FUNCTION
 *   Check if a line starts with a remark marker.  This function
 *   assumes that the remark marker starts on the first character of
 *   the line.
 * INPUTS
 *   o lline_buffer -- the line of text.
 * RESULT
 *   o TRUE -- it starts with a remark marker
 *   o FALSE -- it does not.
 * SOURCE
 */

int
RB_Has_Remark_Marker( char *lline_buffer )
{
    int                 marker;
    int                 found = FALSE;
    char               *cur_char;
    char               *cur_mchar;

    for ( marker = 0;
          ( ( cur_mchar = remark_markers[marker] ) != NULL ) && !found;
          marker++ )
    {
        for ( found = TRUE, cur_char = lline_buffer;
              *cur_mchar && *cur_char && found; cur_mchar++, cur_char++ )
        {
            if ( tolower( *cur_mchar ) != tolower( *cur_char ) )
            {
                found = FALSE;
            }
        }
        if ( found && ! *cur_mchar )
        {
            /* The remark marker equal to the first part
             * of the line
             */
            if ( *cur_char )
            {
               if (isspace( *cur_char ) )
                {
                    /* found a remark marker */
                }
                else
                {
                    /* Some non space character is following the
                     * remark marker... so it is not a proper remark
                     * marker.
                     */
                    found = FALSE;
                }
            }
        }
    }
    return found;
}

/****/
#endif


/****f* Utilities/RB_Alloc_Header [2.01]
 * NAME
 *   RB_Alloc_Header            -- oop
 * SYNOPSIS
 *   struct RB_header *RB_Alloc_Header( void )
 * FUNCTION
 *   allocate the struct RB_header
 * RESULT
 *   struct RB_header *      -- all attributes/pointers set to zero
 * AUTHOR
 *   Koessi
 * SEE ALSO
 *   RB_Free_Header()
 * SOURCE
 */

struct RB_header   *
RB_Alloc_Header( void )
{
    struct RB_header   *new_header;

    if ( ( new_header = malloc( sizeof( struct RB_header ) ) ) != NULL )
    {
        memset( new_header, 0, sizeof( struct RB_header ) );
    }
    else
    {
        RB_Panic( "out of memory! [Alloc Header]\n" );
    }
    return ( new_header );
}

/********/


/****f* Utilities/RB_Free_Header [2.01]
 * NAME
 *   RB_Free_Header             -- oop
 * SYNOPSIS
 *   void RB_Free_Header( struct RB_header *header )
 * FUNCTION
 *   free struct RB_header and associated strings
 * INPUTS
 *   struct RB_header *header -- this one
 * AUTHOR
 *   Koessi
 * SEE ALSO
 *   RB_Alloc_Header(), RB_Close_The_Shop()
 * SOURCE
 */

void
RB_Free_Header( struct RB_header *header )
{
    if ( header )
    {
        if ( header->function_name )
        {
            free( header->function_name );
        }
        if ( header->version )
        {
            free( header->version );
        }
        if ( header->name )
        {
            free( header->name );
        }
        if ( header->unique_name )
        {
            free( header->unique_name );
        }
        if ( header->lines )
        {
            int                 i;

            for ( i = 0; i < header->no_lines; ++i )
            {
                RB_Free( header->lines[i] );
            }
            RB_Free( header->lines );
        }
        free( header );
    }
}

/************/


/****i* Utilities/RB_WordLen [2.01]
 * NAME
 *   RB_WordLen -- like strlen
 * SYNOPSIS
 *   int RB_WordLen( char *str )
 * FUNCTION
 *   get the amount of bytes until next space
 * INPUTS
 *   char *str -- the word
 * RESULT
 *   int -- length of the next word or 0
 * AUTHOR
 *   Koessi
 * SEE ALSO
 *   RB_Find_Header_Name()
 * SOURCE
 */

int
RB_WordLen( char *str )
{
    int                 len;
    char                c;

    for ( len = 0; ( ( c = *str ) != '\0' ) && !isspace( c ) && ( c != '\n' );
          ++str, ++len )
    {
        /* empty */
    }
    return ( len );
}

/*** RB_WordLen ***/


/****i* Utilities/RB_StrDup [2.01]
 * NAME
 *   RB_StrDup
 * SYNOPSIS
 *   char *RB_StrDup( char *str )
 * FUNCTION
 *   duplicate the given string
 * INPUTS
 *   char *str               -- source
 * RESULT
 *   char *                  -- destination
 * AUTHOR
 *   Koessi
 * SOURCE
 */

char               *
RB_StrDup( char *str )
{
    char               *dupstr;
    if ( ( dupstr =
           malloc( ( strlen( str ) + 1 ) * sizeof( char ) ) ) != NULL )
    {
        strcpy( dupstr, str );
    }
    else
    {
        RB_Panic( "out of memory! [StrDup]\n" );
    }
    return ( dupstr );
}

/*** RB_StrDup ***/

/****f* Utilities/RB_Say [2.01]
 * NAME
 *   RB_Say                     -- varargs
 * SYNOPSIS
 *   void RB_Say( char *what, char *why, ... )
 * FUNCTION
 *   Say what's going on.  Goes to stdout.
 * INPUTS
 *   char *format            -- formatstring
 *    ...                    -- parameters
 * AUTHOR
 *   Koessi
 * SOURCE
 */

void
RB_Say( char *format, ... )
{
    va_list             ap;

    if ( course_of_action & DO_TELL )
    {
        va_start( ap, format );
        printf( "%s: ", whoami );
        vprintf( format, ap );
        va_end( ap );
    }
}

/*** RB_Say ***/


void
RB_SetCurrentFile( char *filename )
{
    current_file = filename;
}

char               *
RB_GetCurrentFile(  )
{
    return current_file;
}


/****f* Utilities/RB_Panic [2.01]
 * NAME
 *   RB_Panic -- free resources and shut down
 * SYNOPSIS
 *   void RB_Panic( char *format, char *why, ... )
 * FUNCTION
 *   Print error message.  Frees all resources used by robodoc.
 *   Terminates program.  Output goes to stderr
 * INPUTS
 *   char *format            -- formatstring
 *   ...                     -- parameters
 * AUTHOR
 *   Koessi
 * SOURCE
 */

void
RB_Panic( char *format, ... )
{
    va_list             ap;
    char               *name;

    va_start( ap, format );

    name = RB_GetCurrentFile(  );

    if ( name )
    {
        fprintf( stderr, "%s: FATAL ERROR - %s:%d\n", whoami, name, line_number );
        fprintf( stderr, "%s: %s\n%s: ", whoami, line_buffer, whoami );
    }
    else
    {
        fprintf( stderr, "%s: ", whoami );
    }
    vfprintf( stderr, format, ap );
    fprintf( stderr, "%s: closing down...\n", whoami );
    va_end( ap );
    RB_Close_The_Shop(  );
    exit( EXIT_FAILURE );
}

/*** RB_Panic ***/




/****f* Utilities/RB_Str_Case_Cmp
 * NAME
 *   RB_Str_Case_Cmp
 * SYNOPSIS
 *   int      RB_Str_Case_Cmp(char *, char *)
 *   result = RB_Str_Case_Cmp(s, t)
 * FUNCTION
 *   Compare two strings, regardless of the case of the characters.
 * RESULT
 *    0  s == t
 *   -1  s < t
 *    1  s > t
 * SOURCE
 */

int
RB_Str_Case_Cmp( char *s, char *t )
{
    for ( ; tolower( *s ) == tolower( *t ); s++, t++ )
    {
        if ( *s == '\0' )
        {
            return 0;
        }
    }
    return ( int ) ( tolower( *s ) - tolower( *t ) );
}

/*********/


/****f* Utilities/RB_TimeStamp
 * NAME
 *   RB_TimeStamp -- print a time stamp
 *****
 */

void
RB_TimeStamp( FILE * f )
{
    time_t              ttp;
    char                timeBuffer[255];

    time( &ttp );
    strftime( timeBuffer, 255, "%a %b %d %H:%M:%S %Y\n", localtime( &ttp ) );
    fprintf( f, "%s", timeBuffer );
}

/****f* Utilities/RB_Skip_Whitespace
 * NAME
 *   RB_Skip_Whitespace
 * SYNOPSIS
 *   char * RB_Skip_Whitespace(char *buf)
 * FUNCTION
 *   Skip space and tab chars from the start *buf. This is needed when
 *   searching for indented headers and items. 
 * NOTES
 *   We should extract some info about indentation level and save it to
 *   global variable in order to write out source items (that originate from
 *   indented headers) neatly. 
 * SEE ALSO
 *   RB_Find_Marker, RB_Find_End_Marker, RB_Find_Item, RB_Generate_Item_Body
 * SOURCE
 */

char               *
RB_Skip_Whitespace( char *buf )
{
    char               *c;

    for ( c = buf; *c; c++ )
    {
        switch ( *c )
        {
        case ' ':
        case '\t':
            break;
        default:
            return c;
        }
    }
    return buf;                 /* buf was null */
}

/*** RB_Skip_Whitespace ***/


/****f* Utilities/RB_Free
 * NAME
 *   RB_Free
 * SYNOPSIS
 *   void RB_Free(void *p)
 * FUNCTION
 *   Free allocated memory.
 * SEE ALSO
 *   RB_Calloc, RB_Malloc
 * SOURCE
 */

void
RB_Free( void *p )
{
    if ( p )
    {
        free( p );
    }
    else
    {
        RB_Say( "RB_Free: null memory pointer" );
    }
}

/*** RB_Free ***/

/****f* Utilities/RB_Calloc
 * NAME
 *   RB_Calloc
 * SYNOPSIS
 *   void * RB_Calloc( size_t count, size_t size )
 * SEE ALSO
 *   RB_Malloc, RB_Free
 * SOURCE
 */

void               *
RB_Calloc( size_t count, size_t size )
{
    void               *result;

    result = calloc( count, size );
    if ( !result )
    {
        RB_Panic( "RB_Calloc: Out of memory!" );
    }
    return result;
}

/*** RB_Calloc ***/

/****f* Utilities/RB_Malloc
 * NAME
 *   RB_Malloc
 * SYNOPSIS
 *   void * RB_Malloc( size_t size )
 * SEE ALSO
 *   RB_Calloc, RB_Free
 * SOURCE
 */

void               *
RB_Malloc( size_t size )
{
    void               *result;
    assert( 0 );
    result = malloc( size );
    if ( !result )
    {
        RB_Panic( "RB_Malloc: Out of memory!" );
    }
    return result;
}

/*** RB_Calloc ***/

/****f* Utilities/RB_FputcLatin1ToUtf8
 * NAME
 *   RB_FputcLatin1ToUtf8
 * SYNOPSIS
 *   void RB_FputcLatin1ToUtf8(FILE *fp, int c)
 * BUGS
 *   This wrongly assumes that input is always Latin-1.
 * SOURCE
 */

void
RB_FputcLatin1ToUtf8( FILE * fp, int c )
{
    if ( c < 0x80 )
    {
        if ( fputc( c, fp ) == EOF )
            RB_Panic( "RB_FputcLatin1ToUtf8: write error" );
    }
    else
    {
        if ( fputc( ( 0xC0 | ( c >> 6 ) ), fp ) == EOF )
            RB_Panic( "RB_FputcLatin1ToUtf8: write error" );
        if ( fputc( ( 0x80 | ( c & 0x3F ) ), fp ) == EOF )
            RB_Panic( "RB_FputcLatin1ToUtf8: write error" );
    }
}

/*** RB_FputcLatin1ToUtf8 ***/


/****f* Utilities/RB_CopyFile
 * NAME
 *   RB_CopyFile -- copy a file to another file
 * SYNOPSIS
 *   void RB_CopyFile( char* sourceFileName, char* destinationFileName )
 * RESULT
 *   Program Exit if one of the specified files did not open.
 * SOURCE
 */

void RB_CopyFile( char* sourceFileName, char* destinationFileName )
{
    FILE* source;

    source = fopen( sourceFileName, "r" );
    if ( source ) 
    {
        FILE* dest;
        dest = fopen( destinationFileName, "w" );
        if ( dest ) 
        {
            for (; fgets( line_buffer, MAX_LINE_LEN, source ); )
            {
                fputs( line_buffer, dest );
            }
        }
        else
        {
            fclose( source );
            RB_Panic( "can't open file %s for writing",
                    destinationFileName );
        }
    }
    else
    {
        RB_Panic( "can't open file %s for reading", sourceFileName );
    }
}

/*****/
